# $Id$

package ObsData::Repository;

use strict;
use warnings;
use File::Path;
use ObsData;
use ObsData::Archive;
use Config::IniFiles;

our $CVSID = q$Id$;
our $CVSREV = (q$Revision$ =~ /^Revision: (.*) $/)[0];

my %default_do_if_change = (
    archive_new         => 1,
    archive_time        => 0,
    archive_size        => 0,
    archive_configmtime => 0,

    data_new            => 1,
    data_configmtime    => 0,
);

sub default_do_if_change {
    return %default_do_if_change;
}

=head1 METHODS

=head2 new

=cut

sub new {
    my ($class, $infos, %param) = @_;

    my $or = { %$infos };
    foreach (keys %param) {
        $or->{$_} = $param{$_};
    }
    
    # $infos->
    # obsdata ObsData object
    $or->{obsdata} or die "Error #1de8d015, please insult programmer";
    # datatype search directly fot it
    # obs station, informationnal only
    # statusfile
    # status Config::IniFile object to store extraction info
    
    $or->{status} ||= new Config::IniFiles(
        -file => (($or->{statusfile} && -f $or->{statusfile}) ? $or->{statusfile} : undef),
        # -default => '.', # Is this a good idea
    );
    
    bless($or, $class);

    $or->set_do_if_change;
    $or;
}

sub DESTROY {
    my ($self) = @_;
    $self->loging(0, "Destroy ObsData::Repository for %s", ref($self));
    $self->save_status;
}

sub save_status {
    my ($self) = @_;
    $self->{statusfile} or return;
    $self->{status}->AddSection('.');
    $self->{status}->SetSectionComment('.',
        $ObsData::CVSID,
        $ObsData::Repository::CVSID,
        scalar(localtime),
    );

    $self->{status}->WriteConfig($self->{statusfile});
}

sub loging {
    my $self = shift;
    $self->{obsdata}->loging(@_);
}

sub set_do_if_change {
    my ($self, %settings) = @_;
    
    foreach (keys %default_do_if_change) {
        $self->{do_if_change}{$_} =
            defined($settings{$_}) ? $settings{$_} : $default_do_if_change{$_};
    }
}

package ObsData::Repository::dir;

our @ISA = qw(ObsData::Repository);

sub new {
    my ($class, $infos, %param) = @_;

    my $or = ObsData::Repository->new($infos, %param);
    # dir directory to parse
    # patern glob() over file
    $or->{dir} or die "No dir given";

    bless($or, $class);
}

sub findfile {
    my ($self) = @_;
    my @files;

    if (!-d $self->{dir}) {
        $self->loging(3,
            "directory %s does not exists",
            $self->{dir},
        );
        return;
    }

    my $patern = $self->{patern} || '*';
    foreach my $f (glob("$self->{dir}/$patern")) {

        -f $f or next; # skip no regular files
        
        my $bf = $f;
        $bf =~ s!^\Q$self->{dir}\E/*!!;
        push(@files, $bf);
    }
    @files
}

sub process {
    my ($self) = @_;
    $self->loging(0, "%s() start for %s", (caller(0))[3], $self->{dir});

    foreach my $file ($self->findfile) {
        my $orda = ObsData::Repository::dir::archive->new(
            $self,
            archivefile => $file,
        );
        $orda->get_info;
        if(!$orda->file_need_parse) {
            next;
        }
        $orda->process;
    }
}

package ObsData::Repository::dir::archive;

our @ISA = qw(ObsData::Repository);

sub new {
    my ($class, $infos, %param) = @_;

    my $or = ObsData::Repository->new($infos, %param);
    # dir directory to parse
    # patern glob() over file
    $or->{dir} or die "No dir given";
    $or->{archivefile} or die "No archivefile given";
    
    $or->loging(0,
        "Preparing to parse %s/%s",
         $or->{dir},
         $or->{archivefile}
    );

    bless($or, $class);
}

#sub DESTROY {
#    my ($self) = @_;
#    $self->loging(0, "Destroy ObsData::Repository::archive for %s", ref($self));
#    $self->SUPER::DESTROY();
#}

sub get_info {
    my ($self) = @_;

    $self->{filestats} = [ stat("$self->{dir}/$self->{archivefile}") ];

    if (! @{$self->{filestats}}) {
        $self->loging(4,
            "Cannot get info of %s",
            "$self->{dir}/$self->{archivefile}"
        );
        return 0;
    }

    if ($self->{status}->SectionExists($self->{archivefile})) {
        if (my $oldmtime = $self->{status}->val($self->{archivefile}, 'mtime')) {
            if ($oldmtime == $self->{filestats}[9]) {

            } elsif ($oldmtime > $self->{filestats}[9]) {
                $self->loging(3, 
                    "Wahou, %s is older than my reference: %d => %d", 
                    $oldmtime,
                    $self->{filestats}[9],
                );
            } else {
                $self->{changes}{archive_time} = $self->loging(0, 
                    "%s is more recent %d => %d",
                    $self->{archivefile},
                    $oldmtime,
                    $self->{filestats}[9],
                );
            }
        }

        if (defined(my $oldsize = $self->{status}->val($self->{archivefile}, 'size'))) {
            if ($oldsize != $self->{filestats}[7]) {
                $self->{changes}{archive_size} = $self->loging(0, 
                    "%s size differ %d => %d",
                    $self->{archivefile},
                    $oldsize,
                    $self->{filestats}[7],
                );
            }
        }

        if (defined(my $oldconftime = $self->{status}->val($self->{archivefile}, 'configtime'))) {
            if ($self->{obsdata}->config_mtime > $oldconftime) {
                $self->{changes}{archive_configmtime} = $self->loging(0, 
                    "%s config has changed since last processing %d => %d",
                    $self->{archivefile},
                    $oldconftime,
                    $self->{obsdata}->config_mtime,
                );
            }
        }
    } else {
        $self->{changes}{archive_new} =
            $self->loging(0, "%s is new file", $self->{archivefile});
    }
    return 1;
}

# TODO Define return code for this function, storing result somewhere to
# pass to child.
sub file_need_parse {
    my ($self) = @_;

    my $need = 0;
    foreach (keys %{$self->{changes}}) {
        if (defined($self->{changes}{$_}) && $self->{do_if_change}{$_}) {
            $self->loging(0, "%s => make parse need", $self->{changes}{$_});
            $need++;
        }
    }
    
    $need
}

sub load_archive {
    my ($self) = @_;
    if ($self->{Oarchive}) {
        return 1;
    } else {
        $self->{Oarchive} = ObsData::Archive->new("$self->{dir}/$self->{archivefile}");
        if ($self->{Oarchive}) {
            $self->loging(0,
                "Archive %s/%s properly load",
                $self->{dir}, $self->{archivefile}
            );
            return 1;
        } else {
            $self->loging(4, 
                "Can't handle archive %s/%s: %s, skipping",
                $self->{dir}, $self->{archivefile},
                ObsData::Archive->error
            );
            return 0;
        }
    }
}

sub list_archive {
    my ($self) = @_;
    $self->load_archive or return;
    
    return $self->{Oarchive}->ls;
}

sub parse_archive {
    my ($self) = @_;

    $self->load_archive or return;

    foreach my $datafile ($self->{Oarchive}->ls) {
        $self->loging(0, "archive %s: content %s", $self->{archivefile}, $datafile);
        my $data = ObsData::Repository::dir::archive::data->new(
            $self,
            datafile => $datafile,
        );
        $data->dispatch;
    }
    return 1;
}

sub update_status {
    my ($self) = @_;
    my @filestats = stat("$self->{dir}/$self->{archivefile}");
    if (!$self->{status}->SectionExists($self->{archivefile})) {
        $self->{status}->AddSection($self->{archivefile});
    }
    $self->{status}->newval($self->{archivefile}, 'mtime', $filestats[9]);
    $self->{status}->newval($self->{archivefile}, 'size', $filestats[7]);

    $self->{status}->newval($self->{archivefile}, 'configtime', $self->{obsdata}->config_mtime);
}

# Sub data function:

sub data_lastcheck {
    my ($self, $datafile) = @_;
    my $last_process = $self->{status}->val($self->{archivefile}, "data_processtime_$datafile");
    if ($last_process) {
        $self->loging(0, "%s/%s has been already processed at %s",
            $self->{archivefile},
            $datafile,
            scalar(localtime($last_process)),
        );
    }
    $last_process
}

sub get_data_dest {
    my ($self, $datafile) = @_;

    my ($t, $d) = $self->{obsdata}->build_dest_filename(
        $datafile,
        $self->{datatype}
    );
    $self->loging(0, "%s/%s file, datatype %s, destination %s",
        $self->{archivefile},
        $datafile,
        $t || '(none)',
        $d || '(none)',
    );
    if ($t) {
        $self->{status}->newval($self->{archivefile}, "data_type_$datafile", $t);
    }
    return($t, $d);
}

sub extract_data {
    # $datatype, informationnal only here
    my ($self, $datafile, $datatype, $destfile) = @_;
    $self->load_archive or return 0;
    my $ltime = time;

    $self->{status}->newval(
        $self->{archivefile},
        "data_processtime_$datafile",
        $ltime
    );
    $self->{status}->newval(
        $self->{archivefile},
        "data_processversion_$datafile",
        $ObsData::VERSION
    );

    $self->{status}->SetParameterComment(
        $self->{archivefile}, 
        "data_processtime_$datafile",
        scalar(localtime($ltime)) .
        ", $ObsData::VERSION: $ObsData::CVSREV, $ObsData::Repository::CVSREV",
    );
   
    if (!$destfile) {
        $self->loging(3,
            "%s/%s has no destination",
            $self->{archivefile}, $datafile,
        );
        return 0;
    }
    
    $self->loging(0,
        "Trying to extract %s from %s/%s",
        $destfile,
        $self->{archivefile}, $datafile
    );

    my ($dirname) = $destfile =~ m!(?:(.*)/)?(.*)$!; #! vim syntax sux
    
    if ($dirname && ! -d $dirname) {
        $self->loging(0, "Creating directory %s", $dirname);
        if( ! File::Path::mkpath($dirname)) { 
            $self->loging(
                4, "Can't create directory %s", 
                $dirname
            );
            return 0;
        }
    }

    if ($self->{Oarchive}->extract($datafile, $destfile)) {
        $self->{status}->newval($self->{archivefile}, "data_dest_$datafile", $destfile);
        $self->loging(
            1, "Extraction of %s/%s done as %s",
            $self->{archivefile},
            $datafile,
            $destfile,
        );
            
    } else {
        $self->loging(
            4, "Can't extract %s/%s to %s: %s",
            $self->{archivefile}, $datafile, $destfile,
            $self->{Oarchive}->error,
        );
        return 0;
    }

    return 1;
}

sub process {
    my ($self) = @_;

    foreach my $data ($self->list_archive) {
        if ($self->data_lastcheck($data)) {
            $self->loging(0,
                "%s/%s has allready processed, skipping",
                $self->{archivefile},
                $data
            );
            next;
        }
        $self->extract_data($data, $self->get_data_dest($data));
    }
    
    $self->update_status;

    return 1;
}

1;
